package org.rcisoft.app.pay.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.github.wxpay.sdk.WXPay;
import lombok.extern.slf4j.Slf4j;
import org.jdom.JDOMException;
import org.rcisoft.app.pay.service.WxPayService;
import org.rcisoft.business.cmsActivity.entity.CmsActivity;
import org.rcisoft.business.cmsOrder.dao.CmsOrderRepository;
import org.rcisoft.business.cmsOrder.entity.CmsOrder;
import org.rcisoft.business.memInfo.dao.MemInfoRepository;
import org.rcisoft.business.memInfo.entity.MemInfo;
import org.rcisoft.business.memInfo.service.impl.MemInfoServiceImpl;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.rabbitmq.bean.RabbitMQConfigBean;
import org.rcisoft.core.rabbitmq.config.PayConfig;
import org.rcisoft.core.service.CyRedisService;
import org.rcisoft.core.util.*;
import org.rcisoft.sys.wbac.user.entity.SysUser;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.crypto.BadPaddingException;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.math.BigDecimal;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeUnit;


/**
 * Created with family.
 * author: cy
 * Date: 2021/5/10
 * Time: 4:10 PM
 * description:
 */
@Service
@Slf4j
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
public class WxPayServiceImpl implements WxPayService {


    @Autowired
    private CyRedisService cyRedisServiceImpl;

    @Autowired
    private RedissonClient redissonClient;

    @Autowired
    private RabbitClusterUtil rabbitClusterUtil;

    @Autowired
    private RabbitMQConfigBean rabbitMQConfigBean;

    @Autowired
    private CmsOrderRepository orderInfoRepository;

    @Autowired
    private MemInfoRepository memInfoRepository;
    @Autowired
    private MemInfoServiceImpl memInfoServiceImpl;
    @Autowired
    private UserUtil userUtil;
    @Value("${wx.appId}")
    private String appId;

    @Value("${wx.pay.notifyUrl}")
    private String notifyUrl;
    @Value("${wx.pay.mchId}")
    private String mchId;
    @Value("${wx.pay.key}")
    private String key;
    @Value("${wx.pay.notifyRefundUrl}")
    private String notifyRefundUrl;
    @Value("${wx.pay.certPath}")
    private String certPath;

    private static final String UTF8 = "utf-8";

    @Override
    public JSONObject wxPay(Long orderId, Long activityId, Integer type, HttpServletRequest request) throws IOException, JDOMException {
        //添加实名判断
        userUtil.checkUserAuth("请先进行实名认证!");
        JSONObject json = new JSONObject();
//        try{
            //redis  key: 【order:activity:${aid}:${oid}】 value 【hash】；status -> 待支付 ，可以进行支付
        if (Objects.equals(type, 0)) {
            Object detail = cyRedisServiceImpl.get(RedisCons.ORDER_ACTIVITY + ":" + activityId + ":" + orderId);
            if (detail != null) {
                CmsOrder orderDetail = JSON.parseObject(JSON.toJSONString(detail), CmsOrder.class);
                if (orderDetail.getStatus().equals(0)) {
                    //校验【活动已结束】不允许支付
                    Object activity = cyRedisServiceImpl.get(RedisCons.ACTIVITY_DETAIL + ":" + activityId);
                    if (activity != null) {
                        CmsActivity info = JSON.parseObject(JSON.toJSONString(activity), CmsActivity.class);
                        if (info.getEndTime().before(new Date()))
                            throw new CyServiceException("活动已结束，不允许支付");
                        createWxOrder(request, json, orderDetail);
                    } else
                        throw new CyServiceException("订单支付失败,请稍后重试");
                } else
                    throw new CyServiceException("该订单不是待支付订单,不能支付");
            } else {
                throw new CyServiceException("订单支付失败,请稍后重试");
            }
        } else if (Objects.equals(type, 1)) {
            Object detail = cyRedisServiceImpl.get(RedisCons.ORDER_GOLD + ":" + activityId + ":" + orderId);
            if (detail != null) {
                CmsOrder orderDetail = JSON.parseObject(JSON.toJSONString(detail), CmsOrder.class);
                if (orderDetail.getStatus().equals(0)) {
                    createWxOrder(request, json, orderDetail);
                } else
                    throw new CyServiceException("该订单不是待支付订单,不能支付");
            } else {
                throw new CyServiceException("订单支付失败,请稍后重试");
            }
        }
//        }catch(Exception e){
//            log.error("失败",e);
//            json.put("errMsg", "Failed");
//        }
        return json;
    }

    private void createWxOrder(HttpServletRequest request, JSONObject json, CmsOrder orderDetail) throws IOException, JDOMException {
        //生成的随机字符串
        String nonceStr = Util.getRandomStringByLength(32);
        //商品名称 String body = new String(WXConst.TITLE.getBytes(StandardCharsets.ISO_8859_1), StandardCharsets.UTF_8)
        String body = orderDetail.getName();
        //获取本机的ip地址
        String spbillCreateIp = Util.getIpAddr(request);
        String orderNo = String.valueOf(orderDetail.getBusinessId());
        //支付金额，单位：分，这边需要转成字符串类型，否则后面的签名会失败
        String money = String.valueOf(orderDetail.getPayPrice().multiply(BigDecimal.valueOf(100)).setScale(0, BigDecimal.ROUND_DOWN));
        //用户微信openID
        Object member = cyRedisServiceImpl.get(RedisCons.MEMBER_INFO + ":" + CyUserUtil.getAuthenBusinessId());
        String openid = "";
        if (member != null) {
            SysUser user = JSON.parseObject(JSON.toJSONString(member), SysUser.class);
            openid = user.getWxOpenid();
        }
        Map<String, String> packageParams = new HashMap<>();
        packageParams.put("appid", appId);
        packageParams.put("mch_id", mchId);
        packageParams.put("nonce_str", nonceStr);
        packageParams.put("body", body);
        packageParams.put("out_trade_no", orderNo);//商户订单号
        packageParams.put("total_fee", money);
        packageParams.put("spbill_create_ip", spbillCreateIp);
        packageParams.put("notify_url", notifyUrl);
        packageParams.put("trade_type", WXConst.TRADE_TYPE);
        packageParams.put("openid", openid);
        packageParams.put("attach", String.valueOf(orderDetail.getGoodsId()));

        // 除去数组中的空值和签名参数
        packageParams = PayUtil.paraFilter(packageParams);
        // 把数组所有元素，按照“参数=参数值”的模式用“&”字符拼接成字符串
        String prestr = PayUtil.createLinkString(packageParams);

        //MD5运算生成签名，这里是第一次签名，用于调用统一下单接口
        String mysign = PayUtil.sign(prestr, key, UTF8).toUpperCase();
        log.info("=======================第一次签名：" + mysign + "=====================");
        //拼接统一下单接口使用的xml数据，要将上一步生成的签名一起拼接进去
        String xml = "<xml>" + "<appid>" + appId + "</appid>"
                + "<body>" + body + "</body>"
                + "<mch_id>" + mchId + "</mch_id>"
                + "<nonce_str>" + nonceStr + "</nonce_str>"
                + "<notify_url>" + notifyUrl + "</notify_url>"
                + "<openid>" + openid + "</openid>"
                + "<out_trade_no>" + orderNo + "</out_trade_no>"
                + "<spbill_create_ip>" + spbillCreateIp + "</spbill_create_ip>"
                + "<total_fee>" + money + "</total_fee>"
                + "<trade_type>" + WXConst.TRADE_TYPE + "</trade_type>"
                + "<attach>" + orderDetail.getGoodsId() + "</attach>"
                + "<sign>" + mysign + "</sign>"
                + "</xml>";
        log.info("调试模式_统一下单接口 请求XML数据：" + xml);
        //调用统一下单接口，并接受返回的结果
        String result = PayUtil.httpRequest(WXConst.PAY_URL, "POST", xml);
        log.info("调试模式_统一下单接口 返回XML数据：" + result);
        // 将解析结果存储在HashMap中
        Map map = PayUtil.doXMLParse(result);
        String returnCode = (String) map.get("return_code");//返回状态码
        String resultCode = (String) map.get("result_code");//返回状态码
        //返回给移动端需要的参数
        Map<String, Object> response = new HashMap<>();
        if(Objects.equals(returnCode, "SUCCESS") && Objects.equals(resultCode, "SUCCESS")){
            json.put("success", true);
            // 业务结果
            String prepayId = (String) map.get("prepay_id");//返回的预付单信息
            response.put("nonceStr", nonceStr);
            response.put("package", "prepay_id=" + prepayId);
            Long timeStamp = System.currentTimeMillis() / 1000;
            //这边要将返回的时间戳转化成字符串，不然小程序端调用wx.requestPayment方法会报签名错误
            response.put("timeStamp", timeStamp + "");
            String stringSignTemp = "appId=" + appId + "&nonceStr=" + nonceStr + "&package=prepay_id=" + prepayId+ "&signType=" + WXConst.SIGN_TYPE + "&timeStamp=" + timeStamp;
            //再次签名，这个签名用于小程序端调用wx.requesetPayment方法
            String paySign = PayUtil.sign(stringSignTemp, key, UTF8).toUpperCase();
            log.info("=======================第二次签名：" + paySign + "=====================");
            response.put("paySign", paySign);
            //调用成功，更新状态为支付中
            //redisson key：【order:redis:${oid}】
            //加订单redis锁
            boolean isGetLock = false;
            RLock redisLock = redissonClient.getLock(RedisCons.ORDER_REDIS + orderDetail.getBusinessId());
            try {
                isGetLock = redisLock.tryLock(Long.parseLong(RedisCons.ORDER_INFO_LOCK_WAIT_TIME),
                        Long.parseLong(RedisCons.ORDER_INFO_LOCK_LEASE_TIME), TimeUnit.SECONDS);
                if (isGetLock) {
                    //redis  key: 【order:activity:${aid}:${oid}】 value 【hash】；ttl：-1；hash state-> 【未同步】，status -> 支付中
                    orderDetail.setState(0);
                    orderDetail.setStatus(8);
                    orderDetail.setApplicationStatus("1");
                    if (orderDetail.getType().equals(0))
                        cyRedisServiceImpl.set(RedisCons.ORDER_ACTIVITY +":"+ orderDetail.getGoodsId() + ":" + orderDetail.getBusinessId(),orderDetail);
                    else if (orderDetail.getType().equals(1))
                        cyRedisServiceImpl.set(RedisCons.ORDER_GOLD +":"+ orderDetail.getGoodsId() + ":" + orderDetail.getBusinessId(),orderDetail);
                    //发送 mq，exchange：`order_activity`    route-key：`order_activity_status`   value 【订单】
//                    rabbitClusterUtil.sendMsg(orderDetail, rabbitMQConfigBean.ORDER_EXCHANGE_NAME,rabbitMQConfigBean.ORDER_STATUS_QUEUE_NAME);
                }
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                isGetLock = false;
            } finally {
                redisLock.unlock();
                log.error("释放成功");
            }

        } else {
            json.put("success", false);
            response.put("err_code_des", map.get("err_code_des"));
        }
        response.put("appid", appId);
        json.put("errMsg", "OK");
        json.put("data", response);

    }

    /**
     * 小程序调用wx.requestPayment(OBJECT) 更新订单状态为待支付
     * @param orderId
     * @param activityId
     * @param type
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel updateOrderStatus(Long orderId, Long activityId, Integer type) {
        //加订单redis锁
        boolean isGetLock = false;
        RLock redisLock = redissonClient.getLock(RedisCons.ORDER_REDIS + orderId);
        try {
            isGetLock = redisLock.tryLock(Long.parseLong(RedisCons.ORDER_INFO_LOCK_WAIT_TIME),
                    Long.parseLong(RedisCons.ORDER_INFO_LOCK_LEASE_TIME), TimeUnit.SECONDS);
            if (isGetLock) {
                String redisKey = RedisCons.ORDER_ACTIVITY;
                if (Objects.equals(type, 0))  //活动
                    redisKey = RedisCons.ORDER_ACTIVITY;
                else if (Objects.equals(type, 1))  //金币
                    redisKey = RedisCons.ORDER_GOLD;
                Object detail = cyRedisServiceImpl.get(redisKey +":"+ activityId +":"+ orderId);
                if (detail!=null) {
                    CmsOrder orderDetail = JSON.parseObject(JSON.toJSONString(detail), CmsOrder.class);
                    //redis  key: 【order:activity:${aid}:${oid}】 value 【hash】；ttl：-1；hash state-> 【未同步】，status -> 待支付
                    orderDetail.setState(0);
                    orderDetail.setStatus(0);
                    orderDetail.setApplicationStatus("1");
                    cyRedisServiceImpl.set(redisKey +":"+ orderDetail.getGoodsId() + ":" + orderDetail.getBusinessId(),orderDetail);
                    //发送 mq，exchange：`order_activity`    route-key：`order_activity_status`   value 【订单】
//                    rabbitClusterUtil.sendMsg(orderDetail, rabbitMQConfigBean.ORDER_EXCHANGE_NAME,rabbitMQConfigBean.ORDER_STATUS_QUEUE_NAME);
                } else
                    throw new CyServiceException("订单支付失败，请稍后再试");
            }
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            isGetLock = false;
        } finally {
            redisLock.unlock();
            log.error("释放成功");
        }
        return new CyPersistModel(1);
    }

    //主动查询微信订单状态
    @Override
    public String selectWxOrderStatus(String orderNo)  throws IOException, JDOMException{
        //生成的随机字符串
        String nonceStr = Util.getRandomStringByLength(32);
        Map<String, String> packageParams = new HashMap<>();
        packageParams.put("appid", appId);
        packageParams.put("mch_id", mchId);
        packageParams.put("nonce_str", nonceStr);
        packageParams.put("out_trade_no", orderNo);//商户订单号

        // 除去数组中的空值和签名参数
        packageParams = PayUtil.paraFilter(packageParams);
        // 把数组所有元素，按照“参数=参数值”的模式用“&”字符拼接成字符串
        String prestr = PayUtil.createLinkString(packageParams);

        //MD5运算生成签名，这里是第一次签名
        String mysign = PayUtil.sign(prestr, key, UTF8).toUpperCase();
        log.info("=======================第一次签名：" + mysign + "=====================");
        //拼接统一下单接口使用的xml数据，要将上一步生成的签名一起拼接进去
        String xml = "<xml>" + "<appid>" + appId + "</appid>"
                + "<mch_id>" + mchId + "</mch_id>"
                + "<nonce_str>" + nonceStr + "</nonce_str>"
                + "<out_trade_no>" + orderNo + "</out_trade_no>"
                + "<sign>" + mysign + "</sign>"
                + "</xml>";
        log.info("调试模式_查询订单接口 请求XML数据：" + xml);
        //调用统一下单接口，并接受返回的结果
        String result = PayUtil.httpRequest(WXConst.ORDER_QUERY_URL, "POST", xml);
        log.info("调试模式_查询订单接口 返回XML数据：" + result);
        // 将解析结果存储在HashMap中
        Map map = PayUtil.doXMLParse(result);

        String returnCode = (String) map.get("return_code");//返回状态码
        //返回需要的参数
//        Map<String, Object> response = new HashMap<>()
        if(Objects.equals(returnCode, "SUCCESS")){
            // 业务结果
            String resultCode = (String) map.get("result_code");//返回状态码
            if (Objects.equals(resultCode, "SUCCESS")){
                //SUCCESS--支付成功
                //REFUND--转入退款
                //NOTPAY--未支付
                //CLOSED--已关闭
                //REVOKED--已撤销(刷卡支付)
                //USERPAYING--用户支付中
                //PAYERROR--支付失败(其他原因，如银行返回失败)
                //ACCEPT--已接收，等待扣款
                return (String) map.get("trade_state");
            }
        } else {
//            response.put("return_msg", map.get("return_msg"))
            log.error("订单"+orderNo + "："+map.get("return_msg") + map.get("err_code_des"));
        }
        return "";
    }


    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public void wxNotify(HttpServletRequest request, HttpServletResponse response) throws IOException, JDOMException {
        BufferedReader br = new BufferedReader(new InputStreamReader(request.getInputStream()));
        String line = null;
        StringBuilder sb = new StringBuilder();
        while((line = br.readLine())!=null){
            sb.append(line);
        }
        br.close();
        //sb为微信返回的xml
        String notityXml = sb.toString();
        String resXml = "";
        log.info("接收到的报文：" + notityXml);
        Map map = PayUtil.doXMLParse(notityXml);
        String returnCode = (String) map.get("return_code");
        if("SUCCESS".equals(returnCode)){
            //验证签名是否正确
//            if(PayUtil.verify(PayUtil.createLinkString(map), (String)map.get("sign"), key, UTF8)) {
                /**此处添加自己的业务逻辑代码start**/

                String activityId = (String) map.get("attach");
                String orderId = (String) map.get("out_trade_no");
                String resultCode = (String) map.get("result_code");
                String totalFee = (String) map.get("total_fee");
                String transactionId = (String) map.get("transaction_id");
                //加订单redis锁
                boolean isGetLock = false;
                RLock redisLock = redissonClient.getLock(RedisCons.ORDER_REDIS + orderId);
                try {
                    isGetLock = redisLock.tryLock(Long.parseLong(RedisCons.ORDER_INFO_LOCK_WAIT_TIME),
                            Long.parseLong(RedisCons.ORDER_INFO_LOCK_LEASE_TIME), TimeUnit.SECONDS);
                    if (isGetLock) {
                        String redisKey = RedisCons.ORDER_ACTIVITY; //活动
                        Object detail = cyRedisServiceImpl.get(redisKey + ":" + activityId + ":" + orderId);
                        if (detail == null) {
                            redisKey = RedisCons.ORDER_GOLD; //金币
                            detail = cyRedisServiceImpl.get(redisKey + ":" + activityId + ":" + orderId);
                        }
                        if (detail != null) {
                            CmsOrder orderDetail = JSON.parseObject(JSON.toJSONString(detail), CmsOrder.class);
                            orderDetail.setTransactionId(transactionId);
                            if (!orderDetail.getStatus().equals(8)){
                                resXml = "<xml>" + "<return_code><![CDATA[SUCCESS]]></return_code>"
                                        + "<return_msg><![CDATA[OK]]></return_msg>" + "</xml> ";
                                log.info(resXml);
                                log.info("微信支付回调数据结束");
                                BufferedOutputStream out = new BufferedOutputStream(
                                        response.getOutputStream());
                                out.write(resXml.getBytes());
                                out.flush();
                                out.close();
                                return;
                            }
                            if ("SUCCESS".equals(resultCode)) {
                                String money = String.valueOf(orderDetail.getPayPrice().multiply(BigDecimal.valueOf(100)).setScale(0, BigDecimal.ROUND_DOWN));
                                if (!totalFee.equals(money)) {
                                    log.error("实际付款金额与订单金额不符");
                                    orderDetail.setState(0);
                                    orderDetail.setStatus(0);
                                    orderDetail.setApplicationStatus("1");
                                } else {
                                    orderDetail.setState(0);
                                    orderDetail.setStatus(9);
                                    orderDetail.setApplicationStatus("1");
                                }
                            } else {
                                orderDetail.setState(0);
                                orderDetail.setStatus(0);
                                orderDetail.setApplicationStatus("1");
                            }
                            //redis  key: 【order:activity:${aid}:${oid}】 value 【hash】；ttl：-1；hash state-> 【未同步】，status -> 支付中
                            cyRedisServiceImpl.set(redisKey + ":" + orderDetail.getGoodsId() + ":" + orderDetail.getBusinessId(), orderDetail);
                            //发送 mq，exchange：`order_activity`    route-key：`order_activity_status`   value 【订单】
//                            rabbitClusterUtil.sendMsg(orderDetail, rabbitMQConfigBean.ORDER_EXCHANGE_NAME, rabbitMQConfigBean.ORDER_STATUS_QUEUE_NAME);
                        }
                    }

                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                    isGetLock = false;
                } finally {
                    redisLock.unlock();
                    log.error("释放成功");
                }

                /**此处添加自己的业务逻辑代码end**/

                //通知微信服务器已经支付成功
                resXml = "<xml>" + "<return_code><![CDATA[SUCCESS]]></return_code>"
                        + "<return_msg><![CDATA[OK]]></return_msg>" + "</xml> ";
//            }
        }else{
            resXml = "<xml>" + "<return_code><![CDATA[FAIL]]></return_code>"
                    + "<return_msg><![CDATA[报文为空]]></return_msg>" + "</xml> ";
        }
        log.info(resXml);
        log.info("微信支付回调数据结束");
        BufferedOutputStream out = new BufferedOutputStream(
                response.getOutputStream());
        out.write(resXml.getBytes());
        out.flush();
        out.close();
    }


    //申请退款
    @Override
    public String refundOrder(CmsOrder orderInfo) {
        try {
            log.info("###########执行微信支付退款接口refund方法开始");
            /* 执行微信小程序退款相关业务逻辑 */
            log.info("==========================微信退款开始!!========================");
            Map<String,String> data = new HashMap<>();
            BigDecimal pay = orderInfo.getPayPrice().multiply(BigDecimal.valueOf(100)).setScale(0, BigDecimal.ROUND_DOWN);
            data.put("out_refund_no" , Util.getRandomStringByLength(32));

            data.put("notify_url", notifyRefundUrl); // 根据自己的需求决定需要回调地址
            data.put("transaction_id" , orderInfo.getTransactionId());
            data.put("total_fee" , String.valueOf(pay));
            data.put("refund_fee" , String.valueOf(pay));
            PayConfig config = new PayConfig(appId,mchId,key,certPath);
            WXPay wxpay = new WXPay(config);
            data.put("appid" , appId);
            data.put("mch_id" , mchId);
            data.put("nonce_str" , Util.getRandomStringByLength(32));
            // 除去数组中的空值和签名参数
            data = PayUtil.paraFilter(data);
            // 把数组所有元素，按照“参数=参数值”的模式用“&”字符拼接成字符串
            String prestr = PayUtil.createLinkString(data);
            //MD5运算生成签名，这里是第一次签名
            String sign = PayUtil.sign(prestr, key, UTF8).toUpperCase();
            data.put("sign" , sign);
            Map<String,String> resp = wxpay.refund(data);//获取微信退款返回的结果
            log.info("微信返回信息:\n" + resp);
            String returnCode = resp.get("return_code");   // 返回状态码
            String returnMsg = resp.get("return_msg");     // 返回信息
            if("SUCCESS".equals(returnCode)){
                String resultCode = resp.get("result_code");       //业务结果
                String errCodeDes = resp.get("err_code_des");     //错误代码描述
                if("SUCCESS".equals(resultCode)){
                    log.info("###############退款申请接收成功");
                    return resultCode;
                }else{
                    log.error("提交业务失败errCodeDes:"+errCodeDes);
                    return "FAIL";
                }
            }else{
                // 退款失败
                log.error("微信退款失败returnMsg:"+returnMsg);
                return "FAIL";
            }
        } catch (Exception e) {
            log.error("微信退款失败:{}",e.getMessage());
            return "FAIL";
        }
    }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public void notifyRefundCallBack(HttpServletRequest request, HttpServletResponse response) throws IOException, JDOMException, NoSuchPaddingException, NoSuchAlgorithmException, NoSuchProviderException, InvalidKeyException, IllegalBlockSizeException, BadPaddingException {
        BufferedReader br = new BufferedReader(new InputStreamReader(request.getInputStream()));
        log.info("notifyRefundCallBack---------------" );
        String line = null;
        StringBuilder sb = new StringBuilder();
        while((line = br.readLine())!=null){
            sb.append(line);
        }
        br.close();
        //sb为微信返回的xml
        String notityXml = sb.toString();
        String resXml = "";
        log.info("接收到的报文：" + notityXml);
        Map map = PayUtil.doXMLParse(notityXml);
        String returnCode = (String) map.get("return_code");
        if("SUCCESS".equals(returnCode)){
            //加密信息
            String reqInfo = (String) map.get("req_info");
            String reqInfoXml = PayUtil.getRefundDecrypt(reqInfo,key);
            log.info("-----reqInfoXml :"+reqInfoXml);
            Map reqInfoMap = PayUtil.doXMLParse(reqInfoXml);
                /**此处添加自己的业务逻辑代码start**/

                String orderId = (String) reqInfoMap.get("out_trade_no");
                String refundStatus = (String) reqInfoMap.get("refund_status");
                if ("SUCCESS".equals(refundStatus)) {
                    log.info("退款成功------"+orderId);
                    //加订单redis锁
                    boolean isGetLock = false;
                    RLock redisLock = redissonClient.getLock(RedisCons.ORDER_REDIS + orderId);
                    try {
                        isGetLock = redisLock.tryLock(Long.parseLong(RedisCons.ORDER_INFO_LOCK_WAIT_TIME),
                                Long.parseLong(RedisCons.ORDER_INFO_LOCK_LEASE_TIME), TimeUnit.SECONDS);
                        if (isGetLock) {
                            CmsOrder order = orderInfoRepository.selectById(Long.valueOf(orderId));
                            String redisKey = RedisCons.ORDER_ACTIVITY;
                            if (order.getType().equals(0))  //活动
                                redisKey = RedisCons.ORDER_ACTIVITY;
                            else if (order.getType().equals(1))  //金币
                                redisKey = RedisCons.ORDER_GOLD;
                            Object detail = cyRedisServiceImpl.get(redisKey + ":" + order.getGoodsId() + ":" + orderId);
                            if (detail != null) {
                                CmsOrder orderDetail = JSON.parseObject(JSON.toJSONString(detail), CmsOrder.class);
                                if (!orderDetail.getStatus().equals(2) && !orderDetail.getStatus().equals(3)
                                        && !orderDetail.getStatus().equals(4) && !orderDetail.getStatus().equals(5)){
                                    resXml = "<xml>" + "<return_code><![CDATA[SUCCESS]]></return_code>"
                                            + "<return_msg><![CDATA[OK]]></return_msg>" + "</xml> ";
                                    log.info(resXml);
                                    log.info("微信退款回调数据结束");
                                    BufferedOutputStream out = new BufferedOutputStream(
                                            response.getOutputStream());
                                    out.write(resXml.getBytes());
                                    out.flush();
                                    out.close();
                                    return;
                                }
                                if (orderDetail.getStatus().equals(2) || orderDetail.getStatus().equals(3))
                                    orderDetail.setStatus(6);
                                else
                                    orderDetail.setStatus(7);
                                orderDetail.setApplicationStatus("3");
                                orderDetail.setState(0);
    //                           //redis  key: 【order:activity:${aid}:${oid}】 value 【hash】；ttl：-1；hash state-> 【未同步】，status -> 支付中
                                cyRedisServiceImpl.set(redisKey + ":" + orderDetail.getGoodsId() + ":" + orderDetail.getBusinessId(), orderDetail);
    //                           //发送 mq，exchange：`order_activity`    route-key：`order_activity_status`   value 【订单】
//                                rabbitClusterUtil.sendMsg(orderDetail, rabbitMQConfigBean.ORDER_EXCHANGE_NAME, rabbitMQConfigBean.ORDER_STATUS_QUEUE_NAME);
                            }
                        }
    //
                    } catch (InterruptedException e) {
                        Thread.currentThread().interrupt();
                        isGetLock = false;
                    } finally {
                        redisLock.unlock();
                        log.error("释放成功");
                    }
                } else {
                    log.info("退款失败------"+orderId+":"+refundStatus);
                }


                /**此处添加自己的业务逻辑代码end**/

                //通知微信服务器已经支付成功
                resXml = "<xml>" + "<return_code><![CDATA[SUCCESS]]></return_code>"
                        + "<return_msg><![CDATA[OK]]></return_msg>" + "</xml> ";
//            }
        }else{
            resXml = "<xml>" + "<return_code><![CDATA[FAIL]]></return_code>"
                    + "<return_msg><![CDATA[报文为空]]></return_msg>" + "</xml> ";
        }
        log.info(resXml);
        log.info("微信支付回调数据结束");
        BufferedOutputStream out = new BufferedOutputStream(
                response.getOutputStream());
        out.write(resXml.getBytes());
        out.flush();
        out.close();
    }

    //主动查询微信订单退款状态
    @Override
    public String selectWxOrderRefundStatus(String transactionId)  throws IOException, JDOMException{
        //生成的随机字符串
        String nonceStr = Util.getRandomStringByLength(32);
        Map<String, String> packageParams = new HashMap<>();
        packageParams.put("appid", appId);
        packageParams.put("mch_id", mchId);
        packageParams.put("nonce_str", nonceStr);
        packageParams.put("transaction_id", transactionId);//微信订单号

        // 除去数组中的空值和签名参数
        packageParams = PayUtil.paraFilter(packageParams);
        // 把数组所有元素，按照“参数=参数值”的模式用“&”字符拼接成字符串
        String prestr = PayUtil.createLinkString(packageParams);

        //MD5运算生成签名，这里是第一次签名
        String mysign = PayUtil.sign(prestr, key, UTF8).toUpperCase();
        log.info("=======================第一次签名：" + mysign + "=====================");
        //拼接统一下单接口使用的xml数据，要将上一步生成的签名一起拼接进去
        String xml = "<xml>" + "<appid>" + appId + "</appid>"
                + "<mch_id>" + mchId + "</mch_id>"
                + "<nonce_str>" + nonceStr + "</nonce_str>"
                + "<transaction_id>" + transactionId + "</transaction_id>"
                + "<sign>" + mysign + "</sign>"
                + "</xml>";
        log.info("调试模式_查询订单退款接口 请求XML数据：" + xml);
        //调用查询订单退款接口，并接受返回的结果
        String result = PayUtil.httpRequest(WXConst.REFUND_QUERY_URL, "POST", xml);
        log.info("调试模式_查询订单退款接口 返回XML数据：" + result);
        // 将解析结果存储在HashMap中
        Map map = PayUtil.doXMLParse(result);

        String returnCode = (String) map.get("return_code");//返回状态码
        //返回需要的参数
//        Map<String, Object> response = new HashMap<>()
        if(Objects.equals(returnCode, "SUCCESS")){
            // 业务结果
            String resultCode = (String) map.get("result_code");//返回状态码
            if (Objects.equals(resultCode, "SUCCESS")){
                //退款状态：
                // SUCCESS—退款成功
                // REFUNDCLOSE—退款关闭，指商户发起退款失败的情况。
                // PROCESSING—退款处理中
                // CHANGE—退款异常，退款到银行发现用户的卡作废或者冻结了，导致原路退款银行卡失败，可前往商户平台（pay.weixin.qq.com）-交易中心，手动处理此笔退款。
                // $n为下标，从0开始编号
                return (String) map.get("refund_status_0");
            }
        } else {
//            response.put("return_msg", map.get("return_msg"))
            log.error("订单"+transactionId + "："+map.get("return_msg") + map.get("err_code_des"));
        }
        return "FAIL";
    }

}
